<?php

namespace Drupal\cocoon_media;

use SoapClient;
use SoapHeader;
use StdClass;

/**
 * Cocoon controller.
 */
class CocoonController {

  /**
   * The cocoon domain name.
   *
   * @var string
   */
  public static $domainName = 'use-cocoon.nl';

  /**
   * The number of thumbs per page.
   *
   * @var int
   */
  public $thumbsPerPage = 24;

  /**
   * The sub domain configuration.
   *
   * @var string
   */
  public $subdomain = '';

  /**
   * The user name.
   *
   * @var string
   */
  public $username = '';

  /**
   * API key.
   *
   * @var string
   */
  public $secretkey = '';

  /**
   * CocoonController constructor.
   *
   * @param $sub_domain
   * @param $user_name
   * @param $secret_key
   */
  public function __construct($sub_domain, $user_name, $secret_key) {
    $this->subdomain = $sub_domain;
    $this->username = $user_name;
    // $requestId = $reqId;
    $this->secretkey = $secret_key;
  }

  /**
   * Get the tags from the API.
   *
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   *   TODO check result and add to this documentation.
   *
   * @throws \SoapFault
   */
  public function getTags() {
    try {
      $output = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey)->getTags();
    }
    catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
    }

    return $output;
  }

  /**
   * The SoapClient.
   *
   * @param string $reqId
   *   TODO document this variable.
   * @param string $sub_domain
   *   TODO why is sub_domain a parameter? It is already set in the constructor.
   * @param string $user_name
   *   TODO why is user_name a parameter? It is already set in the constructor.
   * @param string $secret_key
   *   TODO why is secret_key a parameter? It is already set in the constructor.
   *
   * @return \soapClient
   *   TODO check result and add to this documentation.
   *
   * @throws \SoapFault
   */
  public static function soapClient($reqId, $sub_domain, $user_name, $secret_key) {
    $domainName = self::$domainName;
    $requestId = $reqId;
    $wsdl = "https://{$sub_domain}.{$domainName}/webservice/wsdl";

    $hash = sha1($sub_domain . $user_name . $requestId . $secret_key);

    $oAuth = new StdClass();
    $oAuth->username = $user_name;
    $oAuth->requestId = $requestId;
    $oAuth->hash = $hash;

    $oSoapClient = new SoapClient($wsdl, ['exceptions' => 0]);
    $soap_header = new SoapHeader('auth', 'authenticate', $oAuth);
    $oSoapClient->__setSoapHeaders($soap_header);

    return $oSoapClient;
  }

  /**
   * The microtimestamp as string value.
   *
   * @return string
   */
  public function getRequestId() {
    return (string) microtime(TRUE);
  }

  /**
   * TODO Check if this is used.
   * @param $tagId
   *
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   */
  public function getTag($tagId) {
    try {
      $output = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey)->getTag($tagId);
    } catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
    }

    return $output;
  }

  /**
   * @param $tagId
   *
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   */
  public function getFilesByTag($tagId) {
    try {
      $output = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey)->getFilesByTag($tagId);
    } catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
    }

    return $output;
  }

  /**
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   */
  public function getSets() {
    try {
      $output = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey)->getSets();
    }
    catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
    }

    return $output;
  }

  /**
   * @param $setId
   *
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   */
  public function getFilesBySet($setId) {
    try {
      $output = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey)->getFilesBySet($setId);
    }
    catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
    }

    return $output;
  }

  /**
   * @param $fileId
   *
   * @return array
   */
  public function getThumbInfo($fileId) {
    try {
      $subDomain = $this->subdomain;
      $domainName = self::$domainName;
      $url = "https://{$subDomain}.{$domainName}";
      $thumbOrg = 'original';

      $thumbWeb = '500px';

      $noThumb = TRUE;

      $aThumbTypes = $this->getThumbTypes();
      if (array_keys($aThumbTypes)[0]) {
        $thumbWeb = array_keys($aThumbTypes)[0];
      }
      $thumbOrgPath = $aThumbTypes[$thumbOrg]['path'];
      $thumbWebPath = $aThumbTypes[$thumbWeb]['path'];

      $aFile = $this->getFile($fileId);
      if (is_array($aFile)) {
        $filename = $aFile['filename'];
        $extention = strtolower($aFile['extension']);

        if ($extention === 'jpg' ||
          $extention === 'jpeg' ||
          $extention === 'png' ||
          $extention === 'gif' ||
          $extention === 'tiff' ||
          $extention === 'tif' ||
          $extention === 'bmp'
        ) {
          $noThumb = FALSE;
        }

        $fileDim = $aFile['width'] && $aFile['height'] ? $aFile['width'] . ' x ' . $aFile['height'] : '';
        $fileSize = $aFile['size'] ? round($aFile['size'] / 1024) . ' KB' : '';

        if ($aFile['upload_date']) {
          $date = date_create($aFile['upload_date']);
          $fileUploaded = $date;
        }
        else {
          $fileUploaded = '';
        }

        $thumb_ext = 'jpg';
        if ($extention == 'gif' || $extention == 'png' || $extention == 'jpg') {
          $thumb_ext = $extention;
        }
        return [
          'path' => $url . $thumbOrgPath . '/' . $filename . '.' . $extention,
          'web' => !$noThumb ? $url . $thumbWebPath . '/' . $filename . '.' . $thumb_ext : '',
          'ext' => $extention,
          'name' => $filename,
          'dim' => $fileDim,
          'size' => $fileSize,
          'uploaded' => $fileUploaded,
          'domain' => $url,
        ];
      }
    } catch (SoapFault $oSoapFault) {
      $aFile = $oSoapFault;
    }
    return get_object_vars($aFile);
  }

  /**
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   */
  public function getThumbTypes() {
    try {
      $output = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey)->getThumbtypes();
    }
    catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
    }

    return $output;
  }

  /**
   * @param $fileId
   *
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   */
  public function getFile($fileId) {
    try {
      $output = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey)->getFile($fileId);
    }
    catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
      watchdog_exception('file', $oSoapFault);
    }

    return $output;
  }

  /**
   * @return \Drupal\cocoon_media\SoapFault|\Exception
   */
  public function getVersion() {
    try {
      $client = self::soapClient(
        $this->getRequestId(),
        $this->subdomain,
        $this->username,
        $this->secretkey);
      $output = $client->getVersion();
    }
    catch (SoapFault $oSoapFault) {
      $output = $oSoapFault;
    }

    return $output;
  }

  /**
   * @param $errMsg
   *
   * @return false|string
   */
  private function errorResponse($errMsg) {
    return json_encode(['status' => 'error', 'statusMsg' => $errMsg]);
  }

}
